@@ -8,20 +8,18 @@ module Database.Persist.Sqlite
     ( withSqlitePool
     , withSqliteConn
     , createSqlitePool
-    , module Database.Persist
-    , module Database.Persist.GenericSql
+    , module Database.Persist.Sql
     , SqliteConf (..)
+    , runSqlite
+    , wrapConnection
     ) where
 
-import Database.Persist hiding (Entity (..))
-import Database.Persist.Store
-import Database.Persist.EntityDef
-import Database.Persist.GenericSql hiding (Key)
-import Database.Persist.GenericSql.Internal
+import Database.Persist.Sql
 
 import qualified Database.Sqlite as Sqlite
 
 import Control.Monad.IO.Class (MonadIO (..))
+import Control.Monad.Logger (NoLoggingT, runNoLoggingT)
 import Data.List (intercalate)
 import Data.IORef
 import qualified Data.Map as Map
@@ -35,6 +33,7 @@ import Data.Conduit
 import qualified Data.Conduit.List as CL
 import Control.Applicative
 import Data.Int (Int64)
+import Control.Monad ((>=>))
 
 createSqlitePool :: MonadIO m => Text -> Int -> m ConnectionPool
 createSqlitePool s = createSqlPool $ open' s
@@ -50,36 +49,56 @@ withSqliteConn :: (MonadBaseControl IO m, MonadIO m)
 withSqliteConn = withSqlConn . open'
 
 open' :: Text -> IO Connection
-open' s = do
-    conn <- Sqlite.open s
+open' = Sqlite.open >=> wrapConnection
+
+-- | Wrap up a raw 'Sqlite.Connection' as a Persistent SQL 'Connection'.
+--
+-- Since 1.1.5
+wrapConnection :: Sqlite.Connection -> IO Connection
+wrapConnection conn = do
     smap <- newIORef $ Map.empty
     return Connection
-        { prepare = prepare' conn
-        , stmtMap = smap
-        , insertSql = insertSql'
-        , close = Sqlite.close conn
-        , migrateSql = migrate'
-        , begin = helper "BEGIN"
-        , commitC = helper "COMMIT"
-        , rollbackC = ignoreExceptions . helper "ROLLBACK"
-        , escapeName = escape
-        , noLimit = "LIMIT -1"
+        { connPrepare = prepare' conn
+        , connStmtMap = smap
+        , connInsertSql = insertSql'
+        , connClose = Sqlite.close conn
+        , connMigrateSql = migrate'
+        , connBegin = helper "BEGIN"
+        , connCommit = helper "COMMIT"
+        , connRollback = ignoreExceptions . helper "ROLLBACK"
+        , connEscapeName = escape
+        , connNoLimit = "LIMIT -1"
+        , connRDBMS = "sqlite"
         }
   where
     helper t getter = do
         stmt <- getter t
-        _ <- execute stmt []
-        reset stmt
+        _ <- stmtExecute stmt []
+        stmtReset stmt
     ignoreExceptions = E.handle (\(_ :: E.SomeException) -> return ())
 
+-- | A convenience helper which creates a new database connection and runs the
+-- given block, handling @MonadResource@ and @MonadLogger@ requirements. Note
+-- that all log messages are discarded.
+--
+-- Since 1.1.4
+runSqlite :: (MonadBaseControl IO m, MonadIO m)
+          => Text -- ^ connection string
+          -> SqlPersistT (NoLoggingT (ResourceT m)) a -- ^ database action
+          -> m a
+runSqlite connstr = runResourceT
+                  . runNoLoggingT
+                  . withSqliteConn connstr
+                  . runSqlConn
+
 prepare' :: Sqlite.Connection -> Text -> IO Statement
 prepare' conn sql = do
     stmt <- Sqlite.prepare conn sql
     return Statement
-        { finalize = Sqlite.finalize stmt
-        , reset = Sqlite.reset conn stmt
-        , execute = execute' conn stmt
-        , withStmt = withStmt' conn stmt
+        { stmtFinalize = Sqlite.finalize stmt
+        , stmtReset = Sqlite.reset conn stmt
+        , stmtExecute = execute' conn stmt
+        , stmtQuery = withStmt' conn stmt
         }
 
 insertSql' :: DBName -> [DBName] -> DBName -> InsertSqlResult
@@ -100,7 +119,7 @@ insertSql' t cols _ =
 execute' :: Sqlite.Connection -> Sqlite.Statement -> [PersistValue] -> IO Int64
 execute' conn stmt vals = flip finally (liftIO $ Sqlite.reset conn stmt) $ do
     Sqlite.bind stmt vals
-    Sqlite.Done <- Sqlite.step stmt
+    _ <- Sqlite.step stmt
     Sqlite.changes conn
 
 withStmt'
@@ -122,40 +141,41 @@ withStmt' conn stmt vals = bracketP
                 cols <- liftIO $ Sqlite.columns stmt
                 yield cols
                 pull
-showSqlType :: SqlType -> String
+
+showSqlType :: SqlType -> Text
 showSqlType SqlString = "VARCHAR"
 showSqlType SqlInt32 = "INTEGER"
 showSqlType SqlInt64 = "INTEGER"
 showSqlType SqlReal = "REAL"
+showSqlType (SqlNumeric precision scale) = pack $ "NUMERIC(" ++ show precision ++ "," ++ show scale ++ ")"
 showSqlType SqlDay = "DATE"
 showSqlType SqlTime = "TIME"
 showSqlType SqlDayTimeZoned = "TIMESTAMP"
 showSqlType SqlDayTime = "TIMESTAMP"
 showSqlType SqlBlob = "BLOB"
 showSqlType SqlBool = "BOOLEAN"
-showSqlType (SqlOther t) = T.unpack t
+showSqlType (SqlOther t) = t
 
-migrate' :: PersistEntity val
-         => [EntityDef]
+migrate' :: [EntityDef a]
          -> (Text -> IO Statement)
-         -> val
+         -> EntityDef SqlType
          -> IO (Either [Text] [(Bool, Text)])
 migrate' allDefs getter val = do
     let (cols, uniqs) = mkColumns allDefs val
-    let newSql = mkCreateTable False def (cols, uniqs)
+    let newSql = mkCreateTable False def (filter (not . safeToRemove val . cName) cols, uniqs)
     stmt <- getter "SELECT sql FROM sqlite_master WHERE type='table' AND name=?"
     oldSql' <- runResourceT
-             $ withStmt stmt [PersistText $ unDBName table] $$ go
+             $ stmtQuery stmt [PersistText $ unDBName table] $$ go
     case oldSql' of
         Nothing -> return $ Right [(False, newSql)]
-        Just oldSql ->
+        Just oldSql -> do
             if oldSql == newSql
                 then return $ Right []
                 else do
                     sql <- getCopyTable allDefs getter val
                     return $ Right sql
   where
-    def = entityDef val
+    def = val
     table = entityDB def
     go = do
         x <- CL.head
@@ -164,27 +184,35 @@ migrate' allDefs getter val = do
             Just [PersistText y] -> return $ Just y
             Just y -> error $ "Unexpected result from sqlite_master: " ++ show y
 
-getCopyTable :: PersistEntity val
-             => [EntityDef]
+-- | Check if a column name is listed as the "safe to remove" in the entity
+-- list.
+safeToRemove :: EntityDef a -> DBName -> Bool
+safeToRemove def (DBName colName)
+    = any (elem "SafeToRemove" . fieldAttrs)
+    $ filter ((== (DBName colName)) . fieldDB)
+    $ entityFields def
+
+getCopyTable :: [EntityDef a]
              -> (Text -> IO Statement)
-             -> val
-             -> IO [(Bool, Sql)]
+             -> EntityDef SqlType
+             -> IO [(Bool, Text)]
 getCopyTable allDefs getter val = do
     stmt <- getter $ pack $ "PRAGMA table_info(" ++ escape' table ++ ")"
-    oldCols' <- runResourceT $ withStmt stmt [] $$ getCols
+    oldCols' <- runResourceT $ stmtQuery stmt [] $$ getCols
     let oldCols = map DBName $ filter (/= "id") oldCols' -- need to update for table id attribute ?
-    let newCols = map cName cols
+    let newCols = filter (not . safeToRemove def) $ map cName cols
     let common = filter (`elem` oldCols) newCols
-    let id_ = entityID $ entityDef val
+    let id_ = entityID val
     return [ (False, tmpSql)
            , (False, copyToTemp $ id_ : common)
-           , (common /= oldCols, pack dropOld)
+           , (common /= filter (not . safeToRemove def) oldCols, pack dropOld)
            , (False, newSql)
            , (False, copyToFinal $ id_ : newCols)
            , (False, pack dropTmp)
            ]
   where
-    def = entityDef val
+
+    def = val
     getCols = do
         x <- CL.head
         case x of
@@ -196,8 +224,9 @@ getCopyTable allDefs getter val = do
     table = entityDB def
     tableTmp = DBName $ unDBName table `T.append` "_backup"
     (cols, uniqs) = mkColumns allDefs val
-    newSql = mkCreateTable False def (cols, uniqs)
-    tmpSql = mkCreateTable True def { entityDB = tableTmp } (cols, uniqs)
+    cols' = filter (not . safeToRemove def . cName) cols
+    newSql = mkCreateTable False def (cols', uniqs)
+    tmpSql = mkCreateTable True def { entityDB = tableTmp } (cols', uniqs)
     dropTmp = "DROP TABLE " ++ escape' tableTmp
     dropOld = "DROP TABLE " ++ escape' table
     copyToTemp common = pack $ concat
@@ -222,46 +251,44 @@ getCopyTable allDefs getter val = do
 escape' :: DBName -> String
 escape' = T.unpack . escape
 
-mkCreateTable :: Bool -> EntityDef -> ([Column], [UniqueDef]) -> Sql
-mkCreateTable isTemp entity (cols, uniqs) = pack $ concat
+mkCreateTable :: Bool -> EntityDef a -> ([Column], [UniqueDef]) -> Text
+mkCreateTable isTemp entity (cols, uniqs) = T.concat
     [ "CREATE"
     , if isTemp then " TEMP" else ""
     , " TABLE "
-    , T.unpack $ escape $ entityDB entity
+    , escape $ entityDB entity
     , "("
-    , T.unpack $ escape $ entityID entity
+    , escape $ entityID entity
     , " INTEGER PRIMARY KEY"
-    , concatMap sqlColumn cols
-    , concatMap sqlUnique uniqs
+    , T.concat $ map sqlColumn cols
+    , T.concat $ map sqlUnique uniqs
     , ")"
     ]
 
-sqlColumn :: Column -> String
-sqlColumn (Column name isNull typ def _maxLen ref) = concat
+sqlColumn :: Column -> Text
+sqlColumn (Column name isNull typ def _maxLen ref) = T.concat
     [ ","
-    , T.unpack $ escape name
+    , escape name
     , " "
     , showSqlType typ
     , if isNull then " NULL" else " NOT NULL"
     , case def of
         Nothing -> ""
-        Just d -> " DEFAULT " ++ T.unpack d
+        Just d -> " DEFAULT " `T.append` d
     , case ref of
         Nothing -> ""
-        Just (table, _) -> " REFERENCES " ++ T.unpack (escape table)
+        Just (table, _) -> " REFERENCES " `T.append` escape table
     ]
 
-sqlUnique :: UniqueDef -> String
-sqlUnique (UniqueDef _ cname cols _) = concat
+sqlUnique :: UniqueDef -> Text
+sqlUnique (UniqueDef _ cname cols _) = T.concat
     [ ",CONSTRAINT "
-    , T.unpack $ escape cname
+    , escape cname
     , " UNIQUE ("
-    , intercalate "," $ map (T.unpack . escape . snd) cols
+    , T.intercalate "," $ map (escape . snd) cols
     , ")"
     ]
 
-type Sql = Text
-
 escape :: DBName -> Text
 escape (DBName s) =
     T.concat [q, T.concatMap go s, q]
@@ -277,7 +304,7 @@ data SqliteConf = SqliteConf
     }
 
 instance PersistConfig SqliteConf where
-    type PersistConfigBackend SqliteConf = SqlPersist
+    type PersistConfigBackend SqliteConf = SqlPersistT
     type PersistConfigPool SqliteConf = ConnectionPool
     createPoolConfig (SqliteConf cs size) = createSqlitePool cs size
     runPool _ = runSqlPool
